//
// Fan Control
// Copyright 2006 Lobotomo Software 
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "MFDaemon.h"
#import "MFProtocol.h"
#import "MFDefinitions.h"
#import "smc.h"

#define MFApplicationIdentifier     "com.lobotomo.MacProFan"


@implementation MFDaemon

- (id)init
{
    if (self = [super init]) {
        // set sane defaults
        lowerThreshold = 25.0;
        upperThreshold = 50.0;
        baseRpm = 1000;
        maxRpm = MFMaxRpm;
    }
    return self;
}

// store preferences
- (void)storePreferences
{
    CFPreferencesSetValue(CFSTR("baseRpm"), (CFPropertyListRef)[NSNumber numberWithInt:baseRpm],
                          CFSTR(MFApplicationIdentifier), kCFPreferencesAnyUser, kCFPreferencesCurrentHost);
    CFPreferencesSetValue(CFSTR("lowerThreshold"), (CFPropertyListRef)[NSNumber numberWithFloat:lowerThreshold],
                          CFSTR(MFApplicationIdentifier), kCFPreferencesAnyUser, kCFPreferencesCurrentHost);
    CFPreferencesSetValue(CFSTR("upperThreshold"), (CFPropertyListRef)[NSNumber numberWithFloat:upperThreshold],
                          CFSTR(MFApplicationIdentifier), kCFPreferencesAnyUser, kCFPreferencesCurrentHost);
    CFPreferencesSetValue(CFSTR("fahrenheit"), (CFPropertyListRef)[NSNumber numberWithBool:fahrenheit],
                          CFSTR(MFApplicationIdentifier), kCFPreferencesAnyUser, kCFPreferencesCurrentHost);
    CFPreferencesSynchronize(CFSTR(MFApplicationIdentifier), kCFPreferencesAnyUser, kCFPreferencesCurrentHost);
}

// read preferences
- (void)readPreferences
{
    CFPropertyListRef property;
    property = CFPreferencesCopyValue(CFSTR("baseRpm"), CFSTR(MFApplicationIdentifier),
               kCFPreferencesAnyUser, kCFPreferencesCurrentHost);
    if (property) {
        baseRpm = [(NSNumber *)property intValue];
    }
    property = CFPreferencesCopyValue(CFSTR("lowerThreshold"), CFSTR(MFApplicationIdentifier),
               kCFPreferencesAnyUser, kCFPreferencesCurrentHost);
    if (property) {
        lowerThreshold = [(NSNumber *)property floatValue];
    }
    property = CFPreferencesCopyValue(CFSTR("upperThreshold"), CFSTR(MFApplicationIdentifier),
               kCFPreferencesAnyUser, kCFPreferencesCurrentHost);
    if (property) {
        upperThreshold = [(NSNumber *)property floatValue];
    }
    property = CFPreferencesCopyValue(CFSTR("fahrenheit"), CFSTR(MFApplicationIdentifier),
               kCFPreferencesAnyUser, kCFPreferencesCurrentHost);
    if (property) {
        fahrenheit = [(NSNumber *)property boolValue];
    }
}

// this gets called after application start
- (void)start
{
//	// unlike the other fans the PS Fan needs to be put into forced more in order to respond to rpm change request.
//	SMCVal_t	val;
//
//	SMCOpen();
//    strcpy(val.key, SMC_KEY_FAN_CONTROL_MODE);
//    val.bytes[0] = 0;
//    val.bytes[1] = SMC_KEY_FAN3_MASK;
//    val.dataSize = 2;
//	SMCWriteKey(val);									// write out the new fan control key value
//	SMCClose();
	
    [self readPreferences];
    [NSTimer scheduledTimerWithTimeInterval:5.0 target:self selector:@selector(timer:) userInfo:nil repeats:YES];
}

//void readAll()
//{
//	NSLog(@"TA0P = %g", SMCGetTemperature("TA0P"));
//	NSLog(@"TC0C = %g", SMCGetTemperature("TC0C"));
//	NSLog(@"TC0P = %g", SMCGetTemperature("TC0P"));
//	NSLog(@"TC1C = %g", SMCGetTemperature("TC1C"));
//	NSLog(@"TC2C = %g", SMCGetTemperature("TC2C"));
//	NSLog(@"TC3C = %g", SMCGetTemperature("TC3C"));
//	NSLog(@"TCAH = %g", SMCGetTemperature("TCAH"));
//	NSLog(@"TCBH = %g", SMCGetTemperature("TCBH"));
//	NSLog(@"TH0P = %g", SMCGetTemperature("TH0P"));
//	NSLog(@"TH1P = %g", SMCGetTemperature("TH1P"));
//	NSLog(@"TH2P = %g", SMCGetTemperature("TH2P"));
//	NSLog(@"TH3P = %g", SMCGetTemperature("TH3P"));
//	NSLog(@"TM0P = %g", SMCGetTemperature("TM0P"));
//	NSLog(@"TM0S = %g", SMCGetTemperature("TM0S"));
//	NSLog(@"TM1P = %g", SMCGetTemperature("TM1P"));
//	NSLog(@"TM1S = %g", SMCGetTemperature("TM1S"));
//	NSLog(@"TM2P = %g", SMCGetTemperature("TM2P"));
//	NSLog(@"TM2S = %g", SMCGetTemperature("TM2S"));
//	NSLog(@"TM3S = %g", SMCGetTemperature("TM3S"));
//	NSLog(@"TM8P = %g", SMCGetTemperature("TM8P"));
//	NSLog(@"TM8S = %g", SMCGetTemperature("TM8S"));
//	NSLog(@"TM9P = %g", SMCGetTemperature("TM9P"));
//	NSLog(@"TM9S = %g", SMCGetTemperature("TM9S"));
//	NSLog(@"TMAP = %g", SMCGetTemperature("TMAP"));
//	NSLog(@"TMAS = %g", SMCGetTemperature("TMAS"));
//	NSLog(@"TMBS = %g", SMCGetTemperature("TMBS"));
//	NSLog(@"TN0H = %g", SMCGetTemperature("TN0H"));
//	NSLog(@"TS0C = %g", SMCGetTemperature("TS0C"));
//	NSLog(@"Tp0C = %g", SMCGetTemperature("Tp0C"));
//	NSLog(@"Tp1C = %g", SMCGetTemperature("Tp1C"));
//	NSLog(@"Tv0S = %g", SMCGetTemperature("Tv0S"));
//	NSLog(@"Tv1S = %g", SMCGetTemperature("Tv1S"));
//}


double getMaxTemp()
{
//	readAll();
	
	double maxTemp = 0;
	
	char* tempKeys[] = {SMC_KEY_MEM_HEATSINKS};
	int size = sizeof(tempKeys)/sizeof(char*);
   
	int i; for(i = 0; i < size; i++)
	{	
		double curTemp = SMCGetTemperature(tempKeys[i]);
		if(curTemp > maxTemp)
			maxTemp = curTemp;

		//NSLog(@"curTemp %g", curTemp);
	}
	
	//NSLog(@"maxTemp %g", maxTemp);
	return maxTemp;
}

// control loop called by NSTimer
- (void)timer:(NSTimer *)aTimer
{
    double temp;
    int targetRpm;
    int step;

    SMCOpen();

	// get the max temp and use it
	temp = getMaxTemp();
 
    if (temp < lowerThreshold) {
        targetRpm = baseRpm;
    } else if (temp > upperThreshold) {
        targetRpm = maxRpm;
    } else {
        targetRpm = baseRpm + (temp - lowerThreshold) / (upperThreshold - lowerThreshold) * (maxRpm - baseRpm);
    }

    // adjust fan speed in reasonable steps - no need to be too dynamic
    if (currentRpm == 0) {
        step = targetRpm;
    } else {
        step = (targetRpm - currentRpm) / 6;
        if (abs(step) < 20) {
            step = targetRpm - currentRpm;
        }
    }
    targetRpm = currentRpm = currentRpm + step;
    
	 int psRpmTgt = 1200*targetRpm/MFMaxRpm+150;
	 if(psRpmTgt < MFMinRpm)	psRpmTgt = MFMinRpm;
	 //NSLog(@"psRpmTgt %d", psRpmTgt);
	 
    SMCSetFanRpm(SMC_KEY_FAN0_RPM_MIN, targetRpm);	// CPU/Memory Intake Front fan
    SMCSetFanRpm(SMC_KEY_FAN1_RPM_MIN, targetRpm);	// PCIe expansion area front fan
    SMCSetFanRpm(SMC_KEY_FAN2_RPM_MIN, targetRpm);	// CPU/Memory Exhaust back fan
    SMCSetFanRpm(SMC_KEY_FAN3_RPM_TGT, psRpmTgt);	// Power Supply Exhaust back fan
    //SMCSetFanRpm(SMC_KEY_FAN3_RPM_MIN, 500);	// Power Supply Exhaust back fan

    SMCClose();

    // save preferences
    if (needWrite) {
        [self storePreferences];
        needWrite = NO;
    }
}

// accessors
- (int)baseRpm
{
	return baseRpm;
}

- (void)setBaseRpm:(int)newBaseRpm
{
	baseRpm = newBaseRpm;
    needWrite = YES;
}

- (float)lowerThreshold
{
	return lowerThreshold;
}

- (void)setLowerThreshold:(float)newLowerThreshold
{
	lowerThreshold = newLowerThreshold;
    needWrite = YES;
}

- (float)upperThreshold
{
	return upperThreshold;
}

- (void)setUpperThreshold:(float)newUpperThreshold
{
	upperThreshold = newUpperThreshold;
    needWrite = YES;
}

- (BOOL)fahrenheit
{
	return fahrenheit;
}

- (void)setFahrenheit:(BOOL)newFahrenheit
{
	fahrenheit = newFahrenheit;
    needWrite = YES;
}

- (void)temperature:(float *)temperature leftFanRpm:(int *)leftFanRpm rightFanRpm:(int *)rightFanRpm
{
	double temp;

    SMCOpen();
    if (temperature) {
		// read the 2 heat sink temps
		
		// get the max temp and use it
		temp = getMaxTemp();
		
         *temperature = temp;
    }
    if (leftFanRpm) {
        *leftFanRpm = SMCGetFanRpm(SMC_KEY_FAN0_RPM_MIN);
    }
    if (rightFanRpm) {
        *rightFanRpm = SMCGetFanRpm(SMC_KEY_FAN1_RPM_MIN);
    }
    SMCClose();
}

@end
